require "thread"

module Kojak
  # Internal: An extension with central utilities. This module extends top
  # level Kojak module, don't use it directly.
  module Central
    # Global mutex.
    @@mtx = Mutex.new

    # Slow log of investigated methods. Contains list of called names in
    # order of calling.
    @@log = Array.new

    # Global printer.
    @@printer = Printer::RandomlyColorized.new($stderr)

    # Investigated classes.
    @@investigated = {}

    # Internal: Returns slow log array.
    def slowlog
      @@mtx.synchronize { @@log }
    end

    # Internal: List of classes marked for investigation.
    def investigated
      @@mtx.synchronize { @@investigated.values }
    end

    # Internal: Marks given class as investigated.
    #
    # cls - The Class to register.
    #
    # Returns list of investigated classes.
    def mark_for_investigation(cls)
      @@mtx.synchronize { @@investigated[cls.object_id] = cls }
    end

    # Internal: Prints investigation results and writes to slow log.
    #
    # caller - The Caller to print.
    #
    # Returns nothing.
    def print(caller)
      @@mtx.synchronize { @@log << caller[:name] }
      @@printer.puts(Output.new(caller))
    end

    # Internal: Enables investigation of all registered classes.
    def investigate!
      @@mtx.synchronize { @@investigated.values.each(&:investigate!) }
    end
  end
end
