module Kojak
  # Internal: Pretty output string from given caller information.
  class Output < String
    # Public: Constructor. Generates output from given caller.
    def initialize(caller)
      @c = caller
      super("\n" + all.flatten.compact.join("\n") + "\n")
    end

    def all
      [
        header, receiver, location, pid, realtime, arguments, 
        result, err, footer
      ]
    end

    def header
      "=== #{@c[:name]} ".ljust(80, "=")
    end

    def receiver
      "receiver_class=#{@c[:receiver]}"
    end

    def location
      "source_location=#{@c[:location].join(":")}"
    end

    def pid
      "pid=#{@c[:pid]}"
    end

    def realtime
      "real_time=%.5f" % @c[:realtime]
    end

    def arguments
      args = @c[:args]
      return if !args || args.empty?
      ["--- arguments ".ljust(80, '-'), pp(args)]
    end

    def result
      res = @c[:result]
      return if res.nil?
      ["--- result ".ljust(80, '-'), pp(res)]
    end

    def err
      err = @c[:err]
      return if err.nil?
      errname = err.class.name + ": " + err.to_s
      ["--- exception ".ljust(80, '-'), errname]
    end

    def footer
      "=" * 80
    end

    protected

    def pp(x)
      JSON.pretty_generate(x) rescue x.inspect
    end
  end
end
