require 'rainbow'

module Kojak
  # Internal: Special version of thread-safe printer that additionally colorizes
  # the output with one preconfigured color.
  class Printer::Colorized < Printer::Synchronized
    # Public: The String or Symbol with name of the assigned color.
    attr_accessor :color

    # Public: Constructor. Configures printer to write to given output in
    # specified color.
    #
    # out   - The Stream to write data to.
    # color - The String or Symbol with name of the color to use.
    #
    # Returns nothing.
    def initialize(out, color = :white)
      super(out)

      @color   = color
      @rainbow = Rainbow.new
    end

    def write(s, *args)
      super(@rainbow.wrap(s.to_s).color(color), *args)
    end
  end
end
